#include "application.h"

#include <iostream>
#include <cassert>

using namespace std;

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

//maps the display window to the imaginary plane
complex_number application::viewport_to_complex_plane(
    rectangle viewport,  rectangle complex_boundary,
    int x, int y
    )
{
    double viewport_width =  abs(viewport.x1 - viewport.x0);
    double viewport_height = abs(viewport.y1 - viewport.y0);
   
    // preserve original aspect ratio 
    double complex_boundary_width = abs(complex_boundary.x1 - complex_boundary.x0);
    double complex_boundary_height = complex_boundary_width * aspect_ratio;

    complex_number z;

    z.set_components(
        x / viewport_width * complex_boundary_width + complex_boundary.x0,
        y / viewport_height * complex_boundary_height + complex_boundary.y0
    );

    return z;
}   


application::application() : gl_viewer()
{
    cout << "COMMANDS: " << endl;
    cout << "\tc: clears the buffer and rerenders" << endl;
    cout << "\tz: zooms out of the fractal" << endl;

    counter = 0;
}

application::~application()
{
}

void application::init(int argc, char * argv[], int width, int height)
{
    gl_viewer::init(argc,argv,width,height);
    
    //initializes the viewport and the complex boundary
    viewport.set(0,0,this->width,this->height);
    complex_boundary.set(-2.0,-1.2,.50, 1.2);
}

void application::draw_event()
{
    cout << "clearing and rerendering" << endl;
    glColor3f(1.0, 1.0, 1.0);
    
    for (int i = 0; i < this->width; i++)
    {
        for (int j = 0; j < this->height; j++)
        {
            //maps the pixel (i,j) to a coordinate in the complex plane
            complex_number z0 = viewport_to_complex_plane(viewport, complex_boundary, i, j);

            /*
            *
            * YOUR CODE HERE
            *
            */

            glFlush();
        }
    }
}
void application::mouse_event(mouse_button button, 
    mouse_button_state button_state, int x, int y)
{
    // store point if user just depressed the left button
    if ( button == GLUT_LEFT_BUTTON && button_state == GLUT_DOWN)
    {
        //counter used for checking if a user has clicked on two locations in the
        //screen and it will then create the new viewing box
        counter++;
    
        if (counter == 1)
        {
            complex_number z = viewport_to_complex_plane(viewport, complex_boundary, x, y);
            z.get_components( new_complex_boundary.x0, new_complex_boundary.y0 );
        }
        else if (counter == 2)
        {
            complex_number z = viewport_to_complex_plane(viewport, complex_boundary, x, y);
            z.get_components( new_complex_boundary.x1, new_complex_boundary.y1 );

            if (new_complex_boundary.x0 > new_complex_boundary.x1) 
            {
                new_complex_boundary.swap_x();
            }

            if (new_complex_boundary.y0 > new_complex_boundary.y1) 
            {
                new_complex_boundary.swap_y();
            }

            // push current complex boundary into tack
            zoom_history.push_back(complex_boundary);

            // update new complex boundary
            complex_boundary.set(
                new_complex_boundary.x0, new_complex_boundary.y0,
                new_complex_boundary.x1, new_complex_boundary.y1
            );

            glutPostRedisplay();
            counter = 0;
        }
    }
}
void application::keyboard_event(unsigned char key, int x, int y)
{
    // rerender
    if (key == 'c')
        glutPostRedisplay();
    else if (key == 'z')
    {
        // prevents the user zooming out further than the
        // original zoom level
        if (!zoom_history.empty())
        {
            //removes the current view from the stack and reverts to the
            //previous window
            new_complex_boundary = zoom_history[zoom_history.size()-1];
            zoom_history.pop_back();
            complex_boundary.set(new_complex_boundary.x0, new_complex_boundary.y0,
                new_complex_boundary.x1, new_complex_boundary.y1);
            glutPostRedisplay();
        }
    }
}
